/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: Contains the definition of the CProducer class. 
*/



/**
 @file
*/
#ifndef __PRODUCER_H__
#define __PRODUCER_H__

#include "globals.h"
#include "queue.h"

/**
This class owns the producer thread.
It provides an interface to insert tokens into the CQueue::iArray queue.
@see CQueue.
@see CPeriodic.
*/
class CProducer : public CBase
	{
public:
	static CProducer* NewL(CConsoleBase* aConsole,CQueue* aTokens);
	void ConstructL(CConsoleBase* aConsole,CQueue* aTokens);
	static TInt ProdThreadFunc(TAny* aPtr);
	void ResumeProd();
	void Display();
	void Produce();
	~CProducer();
	static TInt InsertFunction(TAny* aPtr);
private:
	CProducer();
private:
	/**
	The pointer to a CQueue object.
	Both CProducer and the CConsumer classes share the same object of the CQueue class.
	@see CConsumer::iQueue.
	*/
	CQueue* iQueue;
	/**
	The producer thread.
	It is invoked periodically by the CProducer::iPeriodicProd object, which is an instance of the CPeriodic class.
	*/
	RThread iProdThread;
	/**
	The CPeriodic object.
	It runs the producer thread (CProducer::iProdThread) at regular intervals.
	@see CProducer::ConstructL().
	*/
	CPeriodic* iPeriodicProd;
	/**
	The user console.
	*/
	CConsoleBase* iConsole;
	};

#endif
